#############################################################################
## Data analysis
## Morphological Processing of Complex and Simple Pseudowords in adults and Older Adults (Lázaro, Ruiz & Illera, 2022)
## Language and Cognition
#############################################################################

# Cleanup workspace 
rm(list=ls())

# Required packages
packages <- c("tidyverse", "plyr", "lme4", "lmerTest", "Hmisc", "effects", "devtools")

# Install required packages. If not, install and load 
for (package in packages) {
    if (!package %in% installed.packages()) {
        install.packages(package, dependencies = TRUE)
    }
    if (!package %in% .packages()) {
        library(package, character.only = TRUE)}
}

# Packages info 
devtools::session_info()$packages %>% 
    dplyr::arrange(desc(date))

############################################################################
## LOAD DATA

## Load data YOUNG adults  
dY <- read.delim("young.txt", header = T, dec = ".") %>%
    as_tibble() %>% 
    mutate(age = "young", 
           pseudowordType = ifelse(item <= 120, "Comp", "Simp"), # Items: 81-120 Complex / 121-140 Simple 
           error = ifelse(RT > 0, 0, 1),  # Errors: RT>0 correct / RT<0 incorrect
           logRT = log(RT)) %>% 
    mutate_if(is.character, as.factor) 

## Load data OLD adults 
dO <- read.delim("old.txt", header = T, dec = ".") %>%
    as_tibble() %>% 
    mutate(age = "old", 
           pseudowordType = ifelse(item <= 120, "Comp", "Simp"), # Items: 81-120 Complex / 121-140 Simple 
           error = ifelse(RT > 0, 0, 1), # Errors: RT>0 correct / RT<0 incorrect
           logRT = log(RT), 
           MoCa = ifelse(MoCa > 21, "High", "Low")) %>% # Dichotomize MoCa: low(MoCa<21) / high(MoCa>21) 
    mutate_if(is.character, as.factor) 

## Observation, errors & outliers 
data <- list(dY, dO) # Create a list with both data frames 

er_ou <- function (data) { # Function for counting errors & outliers 
    obs <- nrow(data)
    errors <- data %>% filter(RT <= 0) %>% nrow() 
    p_errors <- round(errors/obs * 100, 2) 
    outliers <- data %>% filter(RT < 200 & RT > 0) %>% nrow() # Correct RT<200 
    p_outliers <- round(outliers/obs * 100, 2)
    return(list(obs, errors, p_errors, outliers, p_outliers))
}
# apply function to both data frames and fill a matrix with the output 
m <- matrix( unlist ( lapply (data, er_ou) ), nrow = 2, ncol = 5, byrow = T) 
colnames(m) <- c("n", "errors", "% errors", "outliers", "% outliers") # name cols
rownames(m) <- c("dY", "dO") # name rows 
(m) # Matrix with number of observations, errors, % errors, outliers, % outliers 

############################################################################

############################
###### FIRST ANALYSIS ######
############################

## SELECT columns OLD 
dO_S <- dO %>% 
    select("subject", "item", "RT", "ima", "age", "pseudowordType", "error", "logRT")

## Bind YOUNG + OLD
d <- rbind(dY, dO_S) # age as factor with two levels: old / young 

## Trim data for RT analysis: no errors, no RT < 200 
d1_RT <- d %>%  
    filter(RT > 200)

## By subjects RT table
# se function
se <- function(x) sd(x, na.rm = T)/sqrt(length(x))

# group_by: subject, pseudowordType, age
d1_bysubj <- d1_RT %>%  
    group_by(subject, pseudowordType, age) %>% 
    dplyr::summarize(RT = mean(RT)) 

# RT by_subject table 
(by_subj_table <- d1_bysubj %>% 
        group_by(pseudowordType, age) %>% 
        dplyr::summarize(se = se(RT), RT = mean(RT)))

## LMM - RT analysis 
# Contrasts
# sum coding for pseudowordType: comp -1, simp 1
contrasts(d1_RT$pseudowordType) <- c(-1, 1)
# sum coding for age: young -1, old 1
contrasts(d1_RT$age) <- c(-1, 1)

# fit_1: RT model 
fit_1 <- lmer(logRT ~ pseudowordType * age + (pseudowordType|subject) + (1|item),  
              data = d1_RT, control = lmerControl(optimizer = "bobyqa"), REML = F, 
              na.action = na.exclude) 
summary(fit_1) 

## fit_1 plot with effects package 
(ef_1 <- effect("pseudowordType:age", fit_1) %>% 
        as_tibble() %>% 
        dplyr::rename(logRT = fit)) 

# plot_1
(plot_1 <- ef_1 %>% 
    ggplot(aes(pseudowordType, logRT, linetype = age)) + 
    geom_point(size = 2) + 
    geom_errorbar(width = 0, aes(ymax = upper, ymin = lower), lwd = 1.5) + 
    geom_line(aes(group = age), lwd = 1.5) + 
    ggtitle("pseudowordType x age") + 
    theme(plot.title = element_text(hjust = 0.5)) + 
    theme_classic()) 

## GLMM - Error analysis 
# Trimming of outlier responses for error analysis: no 0 < RT < 200 
d1_ERR <- d %>% 
    filter(RT > 200 | RT < 0)

## By subjects error table
# group_by: subject, pseudowordType, age
d1_bysubj_e <- d1_ERR %>%  
    group_by(subject, pseudowordType, age) %>% 
    dplyr::summarize(error = mean(error)) 

# Error by_subject table 
(by_subj_e_table <- d1_bysubj_e %>% 
        group_by(pseudowordType, age) %>% 
        dplyr::summarize(se = se(error), error = mean(error)))

# Contrasts
# sum coding for pseudowordType: comp -1, simp 1
contrasts(d1_ERR$pseudowordType) <- c(-1, 1)
# sum coding for age: young -1, old 1
contrasts(d1_ERR$age) <- c(-1, 1)

## fit_1_e: error model 
fit_1_e <- glmer(error ~ pseudowordType * age + (pseudowordType|subject) + (1|item),  
                 data = d1_ERR, family = "binomial", control = glmerControl(optimizer="bobyqa"), 
                 na.action = na.exclude) 
summary(fit_1_e)

#############################
###### SECOND ANALYSIS ######
#############################

# Subset data frame: only complex pseudowords
# Center imageability: ima 
d1_RT_c <- d1_RT %>% 
    filter(pseudowordType == "Comp") %>% 
    mutate(ima = mean(ima) - ima) 

# Contrasts
# sum coding for age: young -1, old 1
contrasts(d1_RT_c$age) <- c(-1, 1)

# LMM - RT analysis 
# fit_2: RT model
fit_2 <- lmer(logRT ~ ima * age + (1|subject) + (1|item), 
              data = d1_RT_c,control = lmerControl(optimizer = "bobyqa"), REML = F, 
              na.action = na.exclude)
summary(fit_2)

# GLMM - Error analysis 
# Subset data: only complex pseudowords 
# Center imageability 
d1_ERR_c <- d1_ERR %>% 
    filter(pseudowordType == "Comp") %>% 
    mutate(ima = ima - mean(ima)) 

# Contrasts
# sum coding for age: young -1, old 1
contrasts(d1_ERR_c$age) <- c(-1, 1)

# fit_2_e: error model 
fit_2_e <- glmer(error ~ ima * age + (1|subject) + (1|item),  
                 data = d1_ERR_c, family = "binomial", control = glmerControl(optimizer = "bobyqa"), 
                 na.action = na.exclude) 
summary(fit_2_e)

############################
###### THIRD ANALYSIS ######
############################

## Data OLD 
## Complete data OLD adults with age as continuous variable 
dO <- read.delim("old.txt", header = T, dec = ".") %>%
    as_tibble() %>% 
    mutate(age = age - mean(age), # center age 
           cogRes  = cogRes - mean(cogRes), # center cogRes 
           pseudowordType = ifelse(item <= 120, "Comp", "Simp"), 
           error = ifelse(RT > 0, 0, 1), 
           logRT = log(RT), 
           MoCa = ifelse(MoCa > 21, "High", "Low")) %>% # Dichotomize MoCa: low(MoCa<21) / high(MoCa>21) 
    mutate_if(is.character, as.factor) 

## Trim data for RT analysis: no errors, no RT < 200 
d3_RT <- dO %>%  
    filter(RT > 200)

## LMM - RT analysis 
# Contrasts 
# sum coding for pseudowordType: comp -1, simp 1
contrasts(d3_RT$pseudowordType) <- c(-1, 1)
# sum coding for MoCa: low -1, high 1
contrasts(d3_RT$MoCa) <- c(1, -1)

# fit_3A: RT model 
fit_3A <- lmer(logRT ~ age * cogRes * MoCa * pseudowordType + (pseudowordType|subject) + (1|item), 
                data = d3_RT, control = lmerControl(optimizer = "bobyqa"), REML = F, na.action = na.exclude) 
summary(fit_3A)

# fit_3B with age as an additive effect: RT model 
fit_3B<-lmer(logRT ~ age + cogRes * MoCa * pseudowordType + (pseudowordType|subject) + (1|item), 
              data = d3_RT, control = lmerControl(optimizer="bobyqa"), REML = F, na.action = na.exclude) 
summary(fit_3B)
 
## Plot 
## fit_3B plot with effects package 
(ef_3 <- effect("cogRes:pseudowordType", fit_3B) %>% 
    as_tibble() %>% 
    dplyr::rename(logRT = fit) %>% 
    mutate(cogRes = round( mean(d3_RT$cogRes) + cogRes, 0)))

# plot_3B
(plot_3 <- ef_3 %>% 
    ggplot(aes(cogRes, logRT, linetype = pseudowordType)) + 
    stat_smooth(method="lm", se = F, color = "black") + 
    ggtitle("cogRes x pseudowordType") + 
    ylab("logRT") + 
    theme_classic()) 

## GLMM - Error analysis 
# Trimming of outlier responses for error analysis: no 0 < RT < 200 
d3_ERR_O <- dO %>% 
    filter(RT > 200 | RT < 0)

# Contrasts
# sum coding for pseudowordType: comp -1, simp 1
contrasts(d3_ERR_O$pseudowordType) <- c(-1, 1)
# sum coding for age: low -1, high 1
contrasts(d3_ERR_O$MoCa) <- c(1, -1)

# fit_3_e: error model 
fit_3_e <- glmer(error ~ age + cogRes * MoCa * pseudowordType + (pseudowordType|subject) + (1|item), 
                 data = d3_ERR_O, family = "binomial", control = glmerControl(optimizer="bobyqa"), 
                 na.action = na.exclude) 
summary(fit_3_e)

############################################################################
